@@ -129,7 +129,80 @@ def butter_highpass_filter(data, cutoff, fs, order=5):
     y = signal.filtfilt(b, a, data)
     return y
 
+def despike(x, interval = 1800):
+    """
+    Remove spikes from a dataset
+    """
 
+    x_roll = x.rolling(interval,0,center = True).mean().copy()
+
+    x_detrend = (x-x_roll).copy()
+    x_destd = x_detrend.rolling(int(interval/15),0,center = True).std().rolling(interval,0,center = True).mean().copy()
+
+    x[x_detrend>4*x_destd] = np.nan
+    x[x_detrend<-4*x_destd] = np.nan
+    
+    return x   
+# =============================================================================
+# geographical tools    
+# =============================================================================
+
+def cal_dist_dir_on_sphere(longitude, latitude):
+    """
+    function to calculate a series of distances between 
+    coordinate points (longitude and latitude) 
+    of a platform between sequential timesteps
+        
+    Parameters
+    ----------
+    longitude : pd.Series
+         time Series of logitudinal coordinates [deg] of the platform
+    latitude : pd.Series
+        time Series of latitudinal coordinates [deg] of the platform
+            
+    Returns 
+    -------
+    distance : pd.Series
+        distance the platform travelled between each of the timesteps
+    speed : pd.Series
+        speed the platform travelled between each of the timesteps
+    heading : pd.Series
+        direction platform headed between each of the timesteps
+               
+    """
+    
+    # Define the Earths Radius (needed to estimate distance on Earth's sphere)
+    R = 6378137. # [m]
+        
+    # Convert latitude and logitude to radians
+    lon = longitude * np.pi/180.
+    lat = latitude  * np.pi/180.
+        
+    # Calculate the differential of lon and lat between the timestamps 
+    dlon = lon.diff()
+    dlat = lat.diff()
+        
+    # Create a shifted time Series
+    lat_t1 = lat.shift(periods=1)
+    lat_t2 = lat.copy()
+        
+    # Calculate interim stage
+    alpha = np.sin(dlat/2.)**2 + np.cos(lat_t1) * np.cos(lat_t2) * np.sin(dlon/2.)**2
+        
+    distance = 2*R*np.arctan2(np.sqrt(alpha),np.sqrt(1-alpha))#(np.arcsin(np.sqrt(alpha))
+           
+    time_delta = pd.Series((lat.index[1:]-lat.index[0:-1]).seconds, index = lat.index[1::])
+    speed = (distance/time_delta)
+        
+    # Calculate the ships heading
+    arg1 = np.sin(dlon) * np.cos(lat_t2)
+    arg2 = np.cos(lat_t1) * np.sin(lat_t2) -np.sin(lat_t1) * np.cos(lat_t2) * np.cos(dlon)
+        
+    heading = np.arctan2(arg1,arg2) * (-180./np.pi) + 90.0
+    heading[heading<0.0] = heading + 360.
+    heading[heading>360.0] = heading - 360.
+    
+    return distance, speed, heading
 # =============================================================================
 # Wind analysis tools    
 # =============================================================================
